%!

% Functions for performing inversions about a given circle.

%----------------
/invert-point
%----------------
% Input: [x y], the point to invert, and, in brackets, cx, cy, and R, the centre and radius 
% of the circle K in which to invert.
% Output: [x', y'], the image of [x y] under the inversion.
% Eg. "[x y] [cx cy R] invert-point" --> [x' y'].
% Note: We can't invert [cx cy] because it is mapped to the point at infinity.
{
  5 dict begin

  % get input
  aload pop
  /R exch def
  /cy exch def
  /cx exch def
  aload pop
  /y exch def
  /x exch def

  /d x cx sub sq y cy sub sq add def	% (x - cx)^2 + (y - cy)^2
  [
    R sq x cx sub mul d div cx add		% x' = R^2 (x - cx)/d + cx
    R sq y cy sub mul d div cy add		% y' = R^2 (y - cy)/d + cy
  ]

  end
} def

%----------------
/invert-line
%----------------
% Input: [A B C], the homogeneous co-ordinates of the line L which we want to invert, and
% 	[cx cy R], the centre and radius of the circle K in which to invert. 
% Output: If the image is a circle, [x y r], the centre and 
% 	radius of the inverted circle, then a 0 to indicate a circle. 
%	Eg. "[A B C] [cx cy R] invert-line" --> [x y r] 0.
% 	Otherwise the image is the same line, and we output [A B C], the co-ordinates of the 
%	line, then a 1 to indicate a line. 
%	Eg. "[A B C] [cx cy R] invert-line --> [A B C] 1.
{
  5 dict begin

  % get input
  aload pop
  /R exch def
  /cy exch def
  /cx exch def
  aload pop
  /C exch def
  /B exch def
  /A exch def
  /D A cx mul B cy mul C add add def	% Acx + Bcy + C

  % Does L pass through the centre of K?
  D 0 eq	% Acx + Bcy + C = 0?
  {
    % the image is the same line
    [A B C] 1
  } 
  {
    % the image is a circle
    [
      cx A R sq mul 2 div D div sub		% x = cx - AR^2/2D
      cy B R sq mul 2 div D div sub		% y = cy - BR^2/2D
      R sq 2 div D abs div			% R^2/4|D|
      A sq B sq add sqrt mul			% r = R^2/4|D| sqrt(A^2 + B^2)
    ]
    0		% flag output accordingly
  } ifelse
  
  end
} def

%----------------
/invert-circle
%----------------
% Input: [x y r], the centre and radius of the circle C which we want to invert, and
% 	[cx cy R], the centre and radius of the circle K in which to invert. 
% Output: If the image is a circle, [x' y' r'], the centre and 
% 	radius of the inverted circle C', then a 0 to indicate a circle. 
%	Eg. "[x y r] [cx cy R] invert-circle" --> [x' y' r'] 0.
% 	Otherwise the image is a line, and we output [A B C], the homogeneous co-ordinates 
%	of the image Ax + By + C = 0, then a 1 to indicate a line. 
%	Eg. "[x y r] [cx cy R] invert-circle --> [A B C] 1.
{
  5 dict begin

  % get input
  aload pop
  /R exch def
  /cy exch def
  /cx exch def
  aload pop
  /r exch def
  /y exch def
  /x exch def
  /d x cx sub sq y cy sub sq add 
  r sq sub def				% d = (x - cx)^2 + (y - cy)^2 - r^2

  % Does C touch the centre of K?
  d 0 eq	% d = 0?
  {
    % the image is a line
    [
      x cx sub 				% A
      y cy sub				% B
      x cx sub cx mul neg		% -(x - cx)cx
      y cy sub cy mul neg		% -(y - cy)cy
      R sq 2 div neg		% - R^2 / 2
      add add				% C
    ]
    1		% flag output accordingly
  } 
  {
    % the image is a circle
    [
      R sq x cx sub mul d div cx add	% x' = R^2 (x - cx)/d + cx
      R sq y cy sub mul d div cy add	% y' = R^2 (y - cy)/d + cy
      R sq r mul d div 			% r' = R^2(r)/d 
    ]
    0		% flag output accordingly
  } ifelse
  
  end
} def

